﻿/*!
 * MCube UI Library
 * http://www.shellcms.com
 * email:shellcms@gmail.com
 */
MCube.tree = function(config){
	config = config || {};
	this.initialConfig = config;
	if(config.renderTo){
		this.show();
	}
};
MCube.tree.prototype.show = function(position){
	loadData(this);
	function loadData(context){
		var config = context.initialConfig;
		var param = config.param || {};
		$.ajax({
			type: 'post',
			url: config.url,
			data: param,
			dataType: 'json',
			success: function(data){
				show(data,context);
				bindEvents(context);
			},
			error: function(){
				alert(data);
			}
		});
	}
	function bindEvents(context){
		$('.MCube-tree-node',context.tree).bind("click", function(e){
			$('.MCube-tree-node',context.tree).removeClass('MCube-tree-node-selected');
			$(this).removeClass('MCube-tree-node-over');
			$(this).addClass('MCube-tree-node-selected');
			if(context.initialConfig.onClick){
				context.initialConfig.onClick($.data($(this)[0], 'MCube-tree-node'));
			}
		});
		$('.MCube-tree-node',context.tree).bind("mouseover", function(e){
			if(!$(this).hasClass('MCube-tree-node-selected')){
				$(this).addClass('MCube-tree-node-over');
			}
		});
		$('.MCube-tree-node',context.tree).bind("mouseout", function(e){
			$(this).removeClass('MCube-tree-node-over');
		});
		$('.MCube-tree-expanded',context.tree).bind("click", function(e){
			toggleNode($(this).parent());
			return false;
		});
		if(context.initialConfig.checkbox){
			$('.MCube-tree-node-checkbox',context.tree).bind("click", function(e){
				var children = $(this).parent().next();
				var checked = $(this).attr("checked");
				checked ? $('.MCube-tree-node-checkbox',children).attr("checked",true):$('.MCube-tree-node-checkbox',children).attr("checked","");
				var nodes = $('.MCube-tree-node',children);
				for(var i=0;i<nodes.length;i++){
					var data = $.data(nodes.eq(i)[0], 'MCube-tree-node');
					data.checked = checked ? true : false;
					$.data(nodes.eq(i)[0], 'MCube-tree-node',data);
				}
				var node = $(this).parent();
				var data = $.data(node[0], 'MCube-tree-node');
				data.checked = checked ? true : false;
				$.data(node[0], 'MCube-tree-node',data);
				if(context.initialConfig.onCheck){
					context.initialConfig.onCheck($.data(node[0], 'MCube-tree-node'));
				}
				e.stopPropagation();
			});
		}
		function toggleNode(node){
			var hit = $('>span.MCube-tree-hit', node);
			if (hit.length == 0) return;
			
			if (node.children().hasClass('MCube-tree-expanded')){
				collapseNode(node);
			} else {
				expandNode(node);
			}
		}
		function collapseNode(node){
			var expand = $('>span.MCube-tree-expanded', node).removeClass('MCube-tree-expanded').addClass('MCube-tree-collapsed');
			expand.next().removeClass('MCube-tree-folder-open');
			node.next().css('display','none');
		}
		function expandNode(node){
			var expand = $('>span.MCube-tree-collapsed', node).removeClass('MCube-tree-collapsed').addClass('MCube-tree-expanded');
			expand.next().addClass('MCube-tree-folder-open');
			node.next().css('display','block');
		}
	}
	function show(data,context){
		function appendNodes(position,data,deep){
			var ul = $('<ul></ul>').appendTo(position);
			if(deep==1){
				context.tree = ul;
			}
			for(var i=0; i<data.length; i++){
				var li = $('<li></li>').appendTo(ul);
				var node = $('<div class="MCube-tree-node"></div>').appendTo(li);
				$.data(node[0], 'MCube-tree-node', {
					id: data[i].id,
					text: data[i].text,
					attributes: data[i].attributes,
					checked: data[i].checked ? true : false 
				});
				if (data[i].children){
					appendNodes(li,data[i].children,deep+1);
					for(var j=0; j<deep-1; j++){
						$('<span class="MCube-tree-indent"></span>').appendTo(node);
					}			
					$('<span class="MCube-tree-hit MCube-tree-expanded"></span>').appendTo(node);
					$('<span class="MCube-tree-folder MCube-tree-folder-open"></span>').appendTo(node);
				}else{
					for(var j=0; j<deep; j++){
						$('<span class="MCube-tree-indent"></span>').appendTo(node);
					}
					$('<span class="MCube-tree-leaf"></span>').appendTo(node);
				}
				if (context.initialConfig.checkbox){
					$('<input type="checkbox" class="MCube-tree-node-checkbox" >').appendTo(node);
				}
				$('<span class="MCube-tree-node-text">'+data[i].text+'</span>').appendTo(node);
			}
			
		}
		position = (position==undefined ? $('#'+context.initialConfig.renderTo) : position);
		var tree = $('<div class="MCube-tree"></div>').appendTo(position);
		appendNodes(tree,data,1);
	}
}
MCube.tree.prototype.getSelected = function(){
	var r;
	if($('.MCube-tree-node-selected',this.tree).length>0){
		r = $.data($('.MCube-tree-node-selected',this.tree)[0], 'MCube-tree-node');
		return r;
	}else{
		return null;
	}	
}
MCube.tree.prototype.getChecked = function(attribute){
	var r = [];
	for(var i=0;i<$('.MCube-tree-node',this.tree).length;i++){
		if($.data($('.MCube-tree-node',this.tree).eq(i)[0], 'MCube-tree-node').checked){
			if(attribute){
				r.push($.data($('.MCube-tree-node',this.tree).eq(i)[0], 'MCube-tree-node')[attribute]);
			}else{
				r.push($.data($('.MCube-tree-node',this.tree).eq(i)[0], 'MCube-tree-node'));
			}
		}
	}
	return r;
}
